//		+=================================================+
//		|					KbdFunc a1.05                      |
//		|					=============                      |
//		|				Auteur: DOUALOT Nicolas               |
//		|				Email : slubman@ifrance.com           |
//		|         	web   : http://www.ti-arena.fr.st     |
//		+=================================================+

// Le programme fonctionne sur toutes les TI-68k
#define USE_TI92PLUS
#define USE_V200
#define USE_TI89

// Patchs de TIGCC
#define NO_EXIT_SUPPORT

// On inclus les libraires
#include <tigcclib.h>

// Les constantes (version)
// Version 2.05 en hexa!!!
#define version "\x32\x2E\x30\x35"

// Pour le deboguage
//#define debug
#ifdef debug
#define debuga(p) (ST_helpMsg(p))
#else
#define debuga(p)
#endif

// On ajoute le support pour les HW2 (par Kevin KOFLER)
#include "h220xtsr.h"

// On ajoute le TSR  installer en mmoire
#include "kbdhook.h"

// La signature du TSR (12 caractre, complte avec des espaces si ncessaire)
#define signature "evHkKbdFunc "

// La routine d'installation du TSR en mmoire
void _main(void)
{
	// Phrase  la fin de l'installation
	char comment[50];

	// Pas assez de mmoire
	char memory[17]="RAM insuffisante";

	// Handle du bloc allou pour installer le hook
	HANDLE place;

	// Pointeur vers ce bloc
	void *ptsr;

	// Pointeur qui sert  verifier si le TSR est dj install
	void *verif;

	// On va sur HOME
	//cmd_disphome();

	switch(h220xTSR())
	{
		case -3:
			debuga("Hw2patch install");
			break;
		case -2:
			debuga("Installation sur AMS1");
			break;
		case -1:
			debuga("Installation sur HW1");
			break;
		case 0:
			debuga(memory);
			return;
			break;
		default:
			break;
	}

	debuga("point 1");

	// endroit ou est stocke la signature d'un TSR
	verif=EV_hook-16;

	// On vrifie que le programme n'est pas dj install
	// On test la signature en question si un TSR est dj install (EV_hook!=0)
	if (EV_hook)
	{
		// On le vrifie tant que l'on a pas vrifier jusqu'au premier TSR installer
		while (verif)
		{
			// On vrifie la signature
			if (!memcmp(signature, (char*)verif, 12))
			{
				sprintf(comment,"une version de KbdFunc est dj install");
				ST_helpMsg(comment);
				return;
			}

			// On regarde si le TSR respecte la convention de Kevin KOFLER
			if (!memcmp("evHk", (char*)verif,4))
				(unsigned long)verif=*(unsigned long*)(verif-4);
			/* Si un TSR non compatible se trouve dans la liste chaine on arrete la vrification et on tente
			l'installation du programme*/
			else
				break;
		}
	}

	// Taille du TSR+
	//		12 octets pour la signature
	//		4 octets pour enregistrer l'emplacement de l'ancien hook
	//		4 octets pour le pointeur sur la ligne de commande
	//    -2 octets pour le cheksum qui est stock dans le tableau, mais qui n'est pas copi en mmoire
	if (!(place=HeapAllocHigh(sizeof(kbdhook)+12+4-2)))
	{
		ST_helpMsg(memory);
		return;
	}
	debuga("point 2");

	// L'adresse ou l'on peux stocker le TSR
	ptsr=HeapDeref(place);
	ptsr+=0x40000;
	debuga("point 3");

	// On installe le TSR proprement dit en mmoire

	// On copie la signature
	memcpy(ptsr, signature, 12);
	ptsr+=12;
	debuga("point 4");

	// On copie l'adresse du vieux hook
	*((void**)ptsr)=EV_hook;
	ptsr+=4;
	debuga("point 5");

	// On copie le tableau de char contenant le hook
	memcpy(ptsr, kbdhook+2, sizeof(kbdhook)-2);
	debuga("point 6");

	// On bouge le TSR dans une zone particulire de la mmoire
	EX_patch(ptsr, ptsr+sizeof(kbdhook)-3);
	debuga("point 7");

	// On appel le TSR
	ASM_fastcall((ptsr+16));
	debuga("point 8");

	// On sauvegarde la nouvelle valeur du EV_hook
	EV_hook=ptsr;
	debuga("point 9");

	// Affichage du succs de l'installation
	ST_helpMsg("KbdFunc "version" par DOUALOT Nicolas install");
}
