// Le hook d'vnements
// La partie qui tourne en permanence

// Le programme fonctionne sur toutes les TI-68k
#define USE_TI92PLUS
#define USE_V200
#define USE_TI89

// Patchs de TIGCC
#define NO_EXIT_SUPPORT
#define NO_AMS_CHECK

// On inclus les libraires
#include <tigcclib.h>

// Les constantes
// TAG des fichiers
#define _ASM ASM_TAG
#define _DATA DATA_TAG
#define _FUNC FUNC_TAG
#define _OTH OTH_TAG
#define _PIC PIC_TAG
#define _PRGM _FUNC
#define _TEXT TEXT_TAG

// Flags de TEXT_EDIT
#define TE_ACTIVE 0x200

// Pour le deboguage
//#define debug
#ifdef debug
#define debug_ST_helpMsg(p) ({ST_helpMsg(p); ngetchx();})
#define debug_strcpy(d,s) (strcpy(d,s))
#else
#define debug_ST_helpMsg(p)
#define debug_strcpy(d,s)
#endif

// Prototypes des fonctions
TEXT_EDIT *TE_findHomeScreen(void);
BOOL traite_entry(const char*);
BOOL support_type(char*, const char*);
BOOL launch(const char*);

// Mes headers
#include "apd.h"					// Le module qui gre l'apd
#include "associations.h"		// Le module qui lit le fichier avec les associations

CALLBACK void _main(EVENT *ev)
{
	// Marque le dbut de la fonction (Kevin KOFLER)
	extern void *program_start asm("tigcc_compiled.");

	char fname[40], lanceur[18], ext[5], *name=NULL;
	HANDLE h1, h2;
	static TEXT_EDIT *entry=NULL;
	static WINDOW *wind=NULL;

	// On rcupre l'adresse de l'ancien hook
	// Astuce par Kevin KOFLER
	EVENT_HANDLER OldHook=(&program_start)[-1];

	// On rcupre l'adresse des informations de la ligne de commande
	if (!entry) entry=TE_findHomeScreen();

	// On rcupre l'adresse des informations de la fentre home
	if (!wind) wind=FirstWindow;

	// Le programme ne ragit que si l'on appui sur une touche et que Home est la fentre active
	if(ev->Type==CM_KEYPRESS && (wind->Flags & WF_ACTIVE))
	{
		// Appui sur [<>]+[A]
		if(CALCULATOR && ev->extra.Key.Code==KEY_DIAMOND+'A')
		{
			apd();
			ev->Type=CM_IDLE;
		}

		// Appui sur [<>]+[F1...8] (sur les 92+ et v200)
		// 8460=>[<>]+[F1], 8467=>[<>]+[F8] les autres codes sont compris dans l'intervalle
		if(CALCULATOR && (ev->extra.Key.Code>=KEY_DIAMOND+KEY_F1) && (ev->extra.Key.Code<=KEY_DIAMOND+KEY_F8))
		{
			strcpy(fname, "main\\kbdfunc ()");
			fname[12]='1'+((ev->extra.Key.Code)-(KEY_DIAMOND+KEY_F1));
			if (!launch(fname)) ev->Type=CM_IDLE;
			memset(fname, 0, 40);
		}

		// Cas ou la ligne d'entre est active
		if (entry->Flags & TE_ACTIVE)
		{
			// On verouille le bloc de texte de la ligne d'entre!!!
			// On est sure que ce bloc ne bougera pas!!!
			HeapLock(entry->Text.h);

			// On regarde ce qu'il y a dans la ligne d'entre
			name=(char *)HeapDeref(entry->Text.h);

			// Appui sur ["] ou [{] ou [[] ou [(]
			if (ev->extra.Key.Code=='\"' || ev->extra.Key.Code=='{' || ev->extra.Key.Code=='[' || ev->extra.Key.Code=='(')
			{
				// On ajoute le caractre fermant correspondant
				switch (ev->extra.Key.Code)
				{
					case '\"':
						TE_pasteText(entry, "\"\"", 2);
						break;
					case '[':
						TE_pasteText(entry, "[]", 2);
						break;
					case '{':
						TE_pasteText(entry, "{}", 2);
						break;
					case '(':
						TE_pasteText(entry, "()", 2);
						break;
				}

				// On met le curseur au milieu
				TE_select(entry, entry->CursorOffset-1, entry->CursorOffset-1);
				ev->Type=CM_IDLE;
			}

			// Appui sur [ENTER] ou [<>]+[ENTER]
			// + vrification que la chane paut tre un nom de variable
			// + rcupration et vrifification du type de la variable
			// + vrification d'une association avec ce type de variable
			if ((ev->extra.Key.Code==KEY_ENTER || ev->extra.Key.Code==KEY_DIAMOND+KEY_ENTER)
					&& traite_entry(name)
					&& support_type(ext, name)
					&& association(lanceur, ext)
				)
			{
	debug_ST_helpMsg(name);
				// On a donc une variable qui est compatible
				// On construit la ligne de commande
				strcpy(fname, lanceur);
				strcat(fname, "(\"");
				strcat(fname, name);
				strcat(fname, "\")");

debug_ST_helpMsg(fname);
				// On lance le prog et on change le message (pour eviter une erreur)
				if(!launch(fname)) ev->Type=CM_IDLE;

				// On ajoute le nom prog sur le stack
				//push_parse_text(name);
				//h1=HS_popEStack();
				//push_parse_text(fname);
				//h2=HS_popEStack();
				//HeapUnlock(h1);
				//HeapUnlock(h2);
				//HomeStorePair(h1, h2);

				// On slctionne toute la ligne
				TE_select(entry, 0, strlen(name));
				TE_focus(entry);

				// On efface les variables
				memset(fname, 0, 40);
				memset(ext, 0, 5);
				memset(lanceur, 0, 18);
			}

			// On dverouille le bloc de texte de la ligne d'entre!!!
			// On a fini avec le pointeur donc on rend la main au systme!!!
			HeapUnlock(entry->Text.h);
		}
	}

	// On appel l'ancien hook si il y en a un
	if (OldHook)
	{
		(*OldHook)(ev);
	}
}

// Rcupration de la structure TEXT_EDIT de l'EntryLine
// Routine originale par: Samuel STEARLEY
// Conversion en C par: Greg DIETSCHE
TEXT_EDIT *TE_findHomeScreen(void)
{
	register void *a=HomeExecute;

	while(*(unsigned long*)a!=(unsigned long)TE_select)
		a+=2;

	return (TEXT_EDIT*)(unsigned long)(*(unsigned short*)(a-4));
}

// Lancement d'un prog (tir de la FAQ de TIGCC)
BOOL launch(const char *prog)
{
	HANDLE h;
	BOOL r=FALSE;

	// On sauvegarde les registres
	asm ("movem.l %d0-%d7/%a0-%a6, -(%a7)");

	// On sauvegarde le stack
	ESI stack=top_estack;

	// On toknise l'expression (on le rend exploitable quelque soit la langue de la TI)
	push_parse_text(prog);

debug_ST_helpMsg("launch");
	// On met l'exprssion sur le stack
	h=HS_popEStack();
	TRY
		TRY
			// On execute le programme
			NG_execute(h, FALSE);
		FINALLY
			// On renvoie un erreur
			r=FALSE;
			// On libere la mmoire alloue
			HeapFree(h);
			// On vide le tampon
			GKeyFlush();
		ENDFINAL
	ONERR
debug_ST_helpMsg("Lancement impossible");
		// On affiche la boite de dialogue correspondant  l'erreur
		ERD_dialog (errCode, FALSE);
	ENDTRY

	// On restaure le stack
	top_estack=stack;

	// On restaure les registres
	asm ("movem.l (%a7)+, %d0-%d7/%a0-%a6");

	return r;
}

// Traitement de la chane de caractres dans la ligne d'entre
BOOL traite_entry(const char *src)
{
	short i=1;

debug_ST_helpMsg("traite_entry");
	// Si on fourni une source vide on quitte d'emble
	if (!src) return FALSE;
debug_ST_helpMsg(src);

	// On contrle la chane de caractre (contient des lettres ou des chiffres ou '_')
	// Le premier caractre ne peut tre un chiffre
	if ((*src)>'0' && (*src)<'9') return FALSE;

	// Le reste de la chane
	while (*(src+i))
	{
		if (!(isextalnum(*(src+i))) && *(src+i)!='\\') return FALSE;

		// On verifie la longueur
		if (++i>17) return FALSE;
	}

	// Si on arrive ici que la chane peut correspondre  une variable
	return TRUE;
}

// On vrifie que le type est support
//		==0: le type n'est pas support
//		!=0: le type est support
// Stocke l'extension dans dest
BOOL support_type(char *dest, const char *name)
{
//	char nom[18];
	FILE *f;
	short type, i;

debug_ST_helpMsg("support_type");
	// Si il n'y pas de nom ou de destination
	if (!name || !dest) return FALSE;
debug_ST_helpMsg(name);


	// Si f est NULL on n'a pas pu ouvrir le fichier le type n'est pas support
	// Je suppose que c'est  ce niveau que ce produit le bogue lors du premier lancement!!!
	if (!(f=fopen(name, "rb"))) return FALSE;
debug_ST_helpMsg(name);
debug_ST_helpMsg("Ouverture russie");

	// On cherche le type du fichier (sur tout les AMS)
	fseek(f, -1, SEEK_END);
	type=fgetc(f);
debug_ST_helpMsg(name);

	*dest=0;
	switch(type)
	{
		// Les fichiers de type OTH
		case _OTH:
			// Une extension personnalise peux exister si l'avant dernier octet vaut 0
			fseek(f, -2, SEEK_END);
			if (!(fgetc(f)))
			{
				fseek(f, -6, SEEK_END);
				i=0;
				if (!(dest[i]=fgetc(f))) i=-1;
				if (!(dest[i+1]=fgetc(f))) i=-2;
				if (!(dest[i+2]=fgetc(f))) i=-3;
				if (!(dest[i+3]=fgetc(f))) i=-4;
				if (!(dest[i+4]=fgetc(f))) i=-5;
				dest[i]=0;
debug_ST_helpMsg(dest);
			}
			// On peux creer une association avec le type OTH (hditeur hexa par exemple!!!)
			else strcpy(dest, "OTH");
			fclose(f);
			return TRUE;

		// Pour les fichiers PIC, DATA, TEXT on renvoi TRUE si l'option existe dans kbdfuncc
		case _PIC:
			strcpy(dest, "PIC");
			fclose(f);
			return TRUE;
		case _DATA:
			strcpy(dest, "DATA");
			fclose(f);
			return TRUE;
		case _TEXT:
			strcpy(dest, "TEXT");
			fclose(f);
			return TRUE;

		// Pour les fichiers ASM, PRGM et FUNC, ainsi que ceux inconnu on renvoi FALSE
		case _ASM:
		case _PRGM:
		default:
			fclose(f);
			return FALSE;
	}
}
